from fastapi import FastAPI, Request
from fastapi.responses import JSONResponse, FileResponse
from fastapi.staticfiles import StaticFiles
import whois
import datetime
import base64

app = FastAPI()
app.mount("/static", StaticFiles(directory="static"), name="static")

@app.get("/")
async def serve_index():
    return FileResponse("static/index.html")

def check_domain_expiration(domain, contexts):
    try:
        w = whois.whois(domain)
        expiration_date = w.expiration_date
        if isinstance(expiration_date, list):
            expiration_date = expiration_date[0]

        if expiration_date:
            days_left = (expiration_date - datetime.datetime.now()).days
            if days_left > 0:
                return {
                    "domain": domain,
                    "message": f"Domain is active and expires in {days_left} days on {expiration_date.strftime('%Y-%m-%d')}",
                    "context_messages": [],
                    "context_links": [],
                    "context_names": []
                }
            else:
                # Retrieve context-specific messages, links, and names
                context_data = generate_context_message(contexts)
                return {
                    "domain": domain,
                    "message": f"Domain expired on {expiration_date.strftime('%Y-%m-%d')}",
                    "context_messages": context_data["messages"],
                    "context_links": context_data["links"],
                    "context_names": context_data["names"]
                }
        else:
            return {
                "domain": domain,
                "message": "No expiration date found.",
                "context_messages": [],
                "context_links": [],
                "context_names": []
            }
    except Exception as e:
        return {
            "domain": domain,
            "message": f"Error checking domain: {str(e)}",
            "context_messages": [],
            "context_links": [],
            "context_names": []
        }

def generate_context_message(contexts):
    # Define context-specific messages, links, and names
    context_data = {
        "appsec": {
            "name": "Application Security",
            "message": "Application Security Risk: With the domain expired, libraries and dependencies could potentially be replaced with malicious ones, compromising application integrity.",
            "link": "https://example.com/application-security-risk"
        },
        "infrastructure": {
            "name": "Infrastructure",
            "message": "Infrastructure Risk: The expired domain may still be listed as a trusted resource for services like AD/AAD, resource access, and updates, which could be exploited if not managed.",
            "link": "https://example.com/infrastructure-risk"
        },
        "external_users": {
            "name": "External Users",
            "message": "External User Risk: Expired domains may allow someone to acquire the domain, potentially inheriting old email addresses and gaining unwarranted trust or access.",
            "link": "https://example.com/external-user-risk"
        },
        "customer_defense": {
            "name": "Customer Defense",
            "message": "Customer Defense Risk: Expired domains associated with customer email accounts could result in account hijacking if attackers register the expired domain.",
            "link": "https://example.com/customer-defense-risk"
        }
    }

    # Prepare lists for messages, links, and names based on selected contexts
    messages = []
    links = []
    names = []
    for context in contexts:
        if context in context_data:
            messages.append(context_data[context]["message"])
            links.append(context_data[context]["link"])
            names.append(context_data[context]["name"])

    return {"messages": messages, "links": links, "names": names}

@app.post("/upload_domains_base64")
async def upload_domains(request: Request):
    data = await request.json()
    encoded_file = data.get("file")
    contexts = data.get("contexts", [])
    
    # Decode base64 and read lines
    decoded_content = base64.b64decode(encoded_file).decode('utf-8')
    domains = decoded_content.splitlines()
    results = [check_domain_expiration(domain.strip(), contexts) for domain in domains if domain.strip()]
    return JSONResponse(content={"results": results})
